#!/usr/bin/env bash

# mysqldump.sh - create SQL dump of MariaDB/MySQL database.
# Copyright (c) 2022 ge <https://nixhacks.net/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

src_mysqldump() {
    # Create SQL-dump at location __main_target_path via mysqldump(1)
    # Compression is optional.
    # Handle 'mysql' and 'mariadb' URI schemes.
    #
    # Usage: src_mysqldump URI

    log "Run handler ${FUNCNAME[0]}()"

    local uri
    local dst_path

    # shellcheck disable=SC2154
    dst_path="$__main_target_path"
    uri="$1"

    parse_uri "$uri"

    # shellcheck disable=SC2154
    if [[ ! "$scheme" =~ ^(mysql|mariadb)$ ]]; then
        log -p "Error: Wrong URI scheme: $scheme" >&2; exit 1
    fi

    [ -z "$port" ] && port=3306  # Set default MySQL port.

    # shellcheck disable=SC2154
    log "Dumping database '${path##*/}' owned by ${username}@${hostname} ..."

    sqldump="${dst_path}/$(gen_backup_name ".sql")" # Set dump name
    mysqldump_options="${mysqldump_options:-}"

    # See https://dev.mysql.com/doc/refman/8.0/en/environment-variables.html
    # shellcheck disable=SC2154
    export MYSQL_PWD="$password"

    # Set command to execute
    # NOTE! `mysqldump_options` variable must be unquoted!
    # shellcheck disable=SC2086,SC2154
    set --  mysqldump $mysqldump_options \
        --host="$hostname" \
        --port="$port" \
        --user="$username" \
        "${path##*/}"

    log "Run command: $*"

    # shellcheck disable=SC2154
    if "$@" 2>> "$__log_file" > "$sqldump"; then
        # Compress file
        sqldump="$(compress_file "$sqldump")"
        # Append path to 'backups' array
        backups+=("$sqldump")
        log "Dump saved as: $sqldump"
    else
        remove_if_empty "$sqldump"
        handle_error \
            "Error: Something went wrong when executing command:\n\t$*"
    fi

    unset MYSQL_PWD
}
